//constants//
//Size of one tile
var tileSize = 32;
//Size of the border of one tile
var tileBorderSize = 1;
//Color of the border for the tiles
var tileBorderColor = "#000000";
//Color of selected icon
var tileSelectedColor = "#00FF00";

//Width of the map in tiles
var width;
//Height of the map in tiles
var height;

//Map canvas
var mapCanv;
//TypeSelection canvas
var typeCanv;
//Map canvas context
var mapCtx;
//Type Selection canvas context
var typeCtx;

//Width of type selector canvas
var typeWidth = 200;
//Height of type selector canvas
var typeHeight = 300;
//Tiles per row in type tiles
var typeTilesInRow = 4;
//buffer between tiles
var typeTileBuffer;

//numerically stored array of tile types
var tileTypeArray = [];

var selectedTileType = 'floor';

/**
 * Called on the page load. Does initial setup functions.
 */
function load()
{
	mapCanv = document.getElementById('map');
	mapCtx = mapCanv.getContext('2d');
	typeCanv = document.getElementById('typeSelect');
	typeCtx = typeCanv.getContext('2d');
	
	typeTileBuffer = Math.floor((typeWidth - tileSize*typeTilesInRow) / (typeTilesInRow + 1));
	
	loadTypeTiles();
	
	drawTypeTiles();
	
	updateMap();
}

/**
 * loads the image elements of the document into the tile types under tileTypes.<tile>.img
 */
function loadTypeTiles()
{
	for(var key in tileTypes)
	{
		tileTypes[key].img = document.getElementById(key);
		tileTypeArray.push(tileTypes[key]);
	}
}

/**
 * Draws tiles on the type selector
 */
function drawTypeTiles()
{
	var tileBufferSize = typeTileBuffer + tileSize;
	
	typeCtx.clearRect(0, 0, typeWidth, typeHeight);
	
	typeCtx.fillStyle = tileSelectedColor;
	
	var count = 0;
	
	for(var key in tileTypes)
	{
		var xPos = typeTileBuffer + tileBufferSize * (count%typeTilesInRow);
		var yPos = typeTileBuffer + tileBufferSize*Math.floor(count/typeTilesInRow);
		
		if(key == selectedTileType)
		{
			typeCtx.fillRect(xPos-2*tileBorderSize, yPos-2*tileBorderSize,
							 tileSize+4*tileBorderSize, tileSize+4*tileBorderSize);
			console.log(xPos);
			console.log(xPos-tileBorderSize);
		}
		
		typeCtx.drawImage(tileTypes[key].img, xPos, yPos);
		
		count++;
	}
}

/**
 * Draws tiles across the map
 */
function drawTiles()
{
	var fullTileSize = tileSize + 2 * tileBorderSize;
	
	mapCtx.fillStyle = tileBorderColor;
	
	for(var x = 0; x < width; x++)
	{
		for(var y = 0; y < height; y++)
		{
			var xPos = x * fullTileSize;
			var yPos = y * fullTileSize;
			
			mapCtx.fillRect(xPos, yPos, fullTileSize, fullTileSize);
			mapCtx.clearRect(xPos + tileBorderSize, yPos + tileBorderSize, tileSize, tileSize);
		}
	}
}

/**
 * Updates the map canvas width and height to match what was entered in the control bar.
 * Calls the draw tiles method to crete the map tiles after updating size.
 */
function updateMap()
{
	updateWidthHeight();
	
	var map = document.getElementById('map');
	
	var newMapWidth = width * (tileSize + 2 * tileBorderSize);
	var newMapHeight = height * (tileSize + 2 * tileBorderSize);
	
	map.setAttribute('width', newMapWidth);
	map.setAttribute('height', newMapHeight);
	
	drawTiles();
}

/**
 * Updates the width and height variables to match the input
 */
function updateWidthHeight()
{
	width = document.getElementById('levelWidth').value;
	height = document.getElementById('levelHeight').value;
}