//Audio.js - Glauntlet - RIT 2012-2 Preston Johnson
//Creates a pool of all the avaliable lights at runtime.
// Stores unused lights far off screen and allows for fast swapping of lights.

//enum for light types, used to get new lights!!!
 var LIGHT_TYPES = {POINT:0,DIR:1,SPOT:2,AREA:3};

function LightsPool()
{
	
	//make sure an instance hasnt been spawned already and that Engine and Three.js exist
	//if(LightPool!==undefinedENGINE===undefined||THREE===undefined)//for some reason this check is breaking 
	//{
	//	return null;
	//}
		
	
	const MAX_SPOT_LIGHTS = 10;
	const MAX_DIRECTIONAL_LIGHTS = 10;
	const MAX_POINT_LIGHTS = 20;
	const MAX_AREA_LIGHTS = 0;
	
	this.ready = false;
	
	//Light Arrays
	var i;
	this.PointLights = new Array();
	this.SpotLights = new Array();
	this.DirLights = new Array();
	this.AreaLights = new Array();
		
	//all lights will have the following properties added:
	//bool enabled - that determines if they are in use or not
	//int poolIndex - index of where they are stored in the array
	
	//make the point lights
	for(i=0;i<MAX_POINT_LIGHTS;i++)
	{
		this.PointLights[i] = new THREE.PointLight(0x000000);
		
		//new properties
		this.PointLights[i].enabled = false;
		this.PointLights[i].poolIndex = i;
		
		//change some standard properties
		this.PointLights[i].intensity = 0;
		this.PointLights[i].castShadow = true;
		this.PointLights[i].position.x = -9001;
		this.PointLights[i].position.y = -9001;
		this.PointLights[i].position.z = -9001;
		
		this.PointLights[i].shadowCameraNear = 0.1;
		this.PointLights[i].shadowCameraFar = 10000;
		this.PointLights[i].shadowCameraFov = 60;

		this.PointLights[i].shadowBias = 0.005;
		this.PointLights[i].shadowDarkness = 0.75;

		this.PointLights[i].shadowMapWidth = 1024;//SHADOW_MAP_WIDTH;
		this.PointLights[i].shadowMapHeight = 1024;//SHADOW_MAP_HEIGHT;
		
		//ENGINE.currentScene.add(this.PointLights[i]);
	}
		
	//make the spot lights
	for(i=0;i<MAX_SPOT_LIGHTS;i++)
	{
		//new properties
		this.SpotLights[i] = new THREE.SpotLight(0x000000);
		this.SpotLights[i].enabled = false;
		this.SpotLights[i].poolIndex = i;
		
		//change some standard properties
		this.SpotLights[i].intensity = 0;
		this.SpotLights[i].castShadow = false;
		this.SpotLights[i].position.x = -9001;
		this.SpotLights[i].position.y = -9001;
		this.SpotLights[i].position.z = -9001;
		
		//ENGINE.currentScene.add(this.SpotLights[i]);
	}
	
	//make the directional lights
	for(i=0;i<MAX_DIRECTIONAL_LIGHTS;i++)
	{
		//new properties
		this.DirLights[i] = new THREE.DirectionalLight(0x000000);
		this.DirLights[i].enabled = false;
		this.DirLights[i].poolIndex = i;
		
		//change some standard properties
		this.DirLights[i].intensity = 0;
		this.DirLights[i].castShadow = false;
		this.DirLights[i].position.x = -9001;
		this.DirLights[i].position.y = -9001;
		this.DirLights[i].position.z = -9001;
		
		//ENGINE.currentScene.add(this.DirLights[i]);
	}
	
	//make the directional lights
	for(i=0;i<MAX_AREA_LIGHTS;i++)
	{
		//new properties
		this.AreaLights[i] = new THREE.AreaLight(0x000000);
		this.AreaLights[i].enabled = false;
		this.AreaLights[i].poolIndex = i;
		
		//change some standard properties
		this.AreaLights[i].intensity = 0;
		this.AreaLights[i].castShadow = false;
		this.AreaLights[i].position.x = -9001;
		this.AreaLights[i].position.y = -9001;
		this.AreaLights[i].position.z = -9001;
		
		//ENGINE.currentScene.add(this.AreaLights[i]);
	}
	
	
	
	//clear out all the lights on shutdown
	this.DestroyAll = function(){
	
		//Remove all lights
		for(i=0;i<this.PointLights.length;i++)
		{
			if(PointLights[i].enabled)
			{
				ENGINE.currentScene.remove(PointLights[i]);
			}
		}
		
		for(i=0;i<this.DirLights.length;i++)
		{
			if(DirLights[i].enabled)
			{
				ENGINE.currentScene.remove(DirLights[i]);
			}
		}
		
		for(i=0;i<this.AreaLights.length;i++)
		{
			if(AreaLights[i].enabled)
			{
				ENGINE.currentScene.remove(AreaLights[i]);
			}
		}
		
		for(i=0;i<this.SpotLights.length;i++)
		{
			if(SpotLights[i].enabled)
			{
				ENGINE.currentScene.remove(SpotLights[i]);
			}
		}
		
		this.AllLights = null;
		this.PointLights = null;
		this.SpotLights = null;
		this.DirLights = null;
		this.AreaLights = null;
	};
	
	//returns an unused light instance of specified type
	this.GetNewLight = function(lightType)//see LIGHT_TYPES at the top of the doc
	{
		switch(lightType)
		{
			case LIGHT_TYPES.POINT:
				for(j=0;j<this.PointLights.length;j++)
				{
					if(!this.PointLights[j].enabled)
					{
						this.PointLights[j].enabled = true;
						this.PointLights[j].intensity = 1;
						this.PointLights[j].castShadow = false;
						//this.PointLights[j].properties.deferredInitialized = false; // NEEDED HERE TO REDEO SHADOW MAPS AND OTHER DEFERRED SHIT
						return this.PointLights[j];
					}
				}
				return null;
				break;
			default:
				return null;
		}
		
		return null;
	}
	
	this.FreeLight = function(light)
	{
		if(!light.poolIndex||!light.enabled)
		{
			return false;
		}
		
		//ENGINE.currentScene.remove(light);
		light.enabled = false;
		light.intensity = 0;
		light.castShadow = false; // this wont change anything. Already deferred initialized...
		//light.position.x = -9001;
		//light.position.y = -9001;
		//light.position.z = -9001;
	}
	
	this.AddLightFlicker = function(light)
	{
	
	}
	
	this.RemoveLightFlicker = function(light)
	{
	
	}
	
	this.ready = true;
}

//This must be made by the engine on start up
//var LightPool = new LightsPool();
