// Component for Collision

var Collidable = function(){

	this.type = "Collidable";
	this.colliding = false;

	/*
	Initiates the Collidable component to be added to an Object 3D
	*/
	this.init = function(){
		
	};

	/*
		Updates the current state of the component
	*/
	this.update = function() {
	
		var collidables = ENGINE.currentScene.scene.getComponentsInChildren("Collidable");
		//alert(collidables.length);
		
		this.colliding = false;
		
		for(var i = 0; i < collidables.length; i++)
		{
			if(collidables[i] != this)
			{
				if(this.objectCollision(collidables[i].gameObject))
				{
					this.colliding = true;
					//console.log("colliding");
				}
			}
		}

	};

	/*
		Removes this component
	*/
	this.destroy = function() {
		
	};

	/*
		This method uses the radius as width/height and Tests if the passed in object is colliding with the object3D this component is assigned to. Returns true or false.
		@param {...object3D} obj - the object to test collision on
	*/
	this.objectCollisionAsSquare = function(obj) {

		if( ((2 * (this.gameObject.boundingRadius * this.gameObject.boundingRadiusScale)) + this.gameObject.position.x >= obj.position.x) 
			&& (this.gameObject.position.x <= obj.position.x + 2 * (obj.boundingRadius * obj.boundingRadiusScale)) 
			
			&& ((2 * (this.gameObject.boundingRadius * this.gameObject.boundingRadiusScale)) + this.gameObject.position.y >= obj.position.y)
			&& (this.gameObject.position.y <= obj.position.y + 2 * (obj.boundingRadius * obj.boundingRadiusScale))
			
			&&( (2 * (this.gameObject.boundingRadius * this.gameObject.boundingRadiusScale)) + this.gameObject.position.z >= obj.position.z) 
			&& (this.gameObject.position.z <= obj.position.z + 2 * (obj.boundingRadius * obj.boundingRadiusScale)))
			{
				return true;
			}
		else
			return false;
			
	};

	this.objectCollision = function(obj) {

		// Calculate difference between centres
		var distX = this.gameObject.position.x - obj.position.x;
		var distY = this.gameObject.position.y - obj.position.y;
		var distZ = this.gameObject.position.z - obj.position.z;
		// Get distance with Pythagoras
		var dist = Math.sqrt((distX * distX) + (distY * distY) + (distZ * distZ));
		return dist <= ((this.gameObject.boundRadius * this.gameObject.boundRadiusScale) + (obj.boundRadius * obj.boundRadiusScale));
	};


	/*
		Tests if the passed in 3D vector intersects the passed in object3D
		@param {...Vector3} line - the line that comes from the object this component is assigned to.
		@param {...object3D} obj - the object3D to test line collision on
	*/
	this.lineCollision = function(vector, obj) {
		
		var newX = vector.x + this.gameObject.rotation.x;
		var newY = vector.y + this.gameObject.rotation.y;
		var newZ = vector.z + this.gameObject.rotation.z;
		
		if(newX >= obj.position.x && newX <= obj.position.x + obj.boundingRadius
			&& newY >= obj.position.y && newY <= obj.position.y + obj.boundingRadius
			&& newZ >= obj.position.z && newZ <= obj.position.z + obj.boundingRadius)
				return true;
		else
			return false;
	};
}
