// Component for magic collision detection

/*
	Constructor for the projectile component
	@param {...gameObject} launch - the game object that fired the projectile
	@param {...Vector3} dmg - the damage of the projectile
	@param {...Vector3} vel - the direction of the projectile
*/
var MagicObj = function(user, dmg){

	this.type = "MagicObj";
	this.user = user;
	this.damage = dmg;
		

	/*
	Initiates the projectile component to be added to an Object 3D
	*/
	this.init = function(){
		console.log("Magic object created by "+this.user+" and it will do "+this.damage+" damage.");
	};

	/*
		Updates the current state of the component
	*/
	this.update = function(gameTime) {
		
		var cm = ENGINE.currentScene.scene.getChildByName("collisionManager");
		
		if(cm)
			cm = cm.getComponent("collisionManager");
		else
			console.log("Could not find Collision Manager");
		
		//var isOnScreen = ENGINE.currentScene.mainCamera.getComponent("CAMERTRACKER_CP").IsOnScreen(this.gameObject);
		//Check if its on screen, fine. If not, destroy it
		
		//Get all gameobjects that have an enemyChar component
				//Check to see if they are in the bounds of the camera
					//Damage those that are
		if(this.gameObject.getComponent("collider") != null)
		{
			//console.log("In if statement: "+this.gameObject.getComponent("collider").colliding);
			if(this.gameObject.getComponent("collider").colliding == true)
			{
				var collider = this.gameObject.getComponent("collider");
				for(var i = 0; i < collider.contacts.length; i++)
				{
					var contact = collider.contacts[i];
					
					//if the collision is not from another player/enemy
					if(contact.otherCollider.gameObject.getComponent("enemyChar"))
					{
						contact.otherCollider.gameObject.getComponent("enemyChar").takeDamage(dmg);
						console.log(dmg);
					}
					
				}
				//console.log("Magic object destroyed");
				
			}
			for(j=0;j<this.gameObject.children.length;j++)
			{
				if(this.gameObject.children[j] instanceof THREE.PointLight)
				{
					ENGINE.LightPool.FreeLight(this.gameObject.children[j]);
					break;
				}
			}
			this.destroy();
		}		
	};

	/*
		Removes this component
	*/
	this.destroy = function() {
		//console.log("bullet destroyed");
		
		this.gameObject.removeComponent(this.type);
		ENGINE.currentScene.remove(this.gameObject);
	};

	

}