// Component for Animator

var Animator = function () {

	this.isPlaying = false;
	this.nowPlaying = "";

	this.weight = 0.0;
	this.WEIGHT_INC = 0.03;

	this.currSkin = 0;
	this.nextSkin = 1;

	this.animType = "MORF";		// or "KEYF"


	this.type = "ANIMATOR_CP";			// CHANGE THIS !


	/*
	   Initializes the Animator component to be added to an Object3D
	 */
	this.init = function(){
		this.progress = 0;
	};

	/*
	   Sets the skin used to move the morph target influences through
	 */
	this.setCollada = function (collada) {
		this.collada = collada;
		this.skin = this.collada.skins[0];

		if (this.skin != null) {
			this.animType = "MORF";
		} else {
			this.animType = "KEYF";

			var animHandler = THREE.AnimationHandler;

			this.animations = this.collada.animations;
			this.kfAnimationsLength = 1;
			this.kfAnimations = [ ];

			for ( var i = 0; i < this.kfAnimationsLength; ++i ) {
				var animation = this.animations[ i ];
				animHandler.add( animation );

				var kfAnimation = new THREE.KeyFrameAnimation( animation.node, animation.name );
				kfAnimation.timeScale = 1;
				this.kfAnimations.push( kfAnimation );
			}

			this.restartKFAnimation ();
		}
	};

	this.restartKFAnimation = function () {

		for ( var i = 0; i < this.kfAnimationsLength; ++i ) {

			var animation = this.kfAnimations[i];

			for ( var h = 0, hl = animation.hierarchy.length; h < hl; h++ ) {

				var keys = animation.data.hierarchy[ h ].keys;
				var sids = animation.data.hierarchy[ h ].sids;
				var obj = animation.hierarchy[ h ];

				if ( keys.length && sids ) {

					for ( var s = 0; s < sids.length; s++ ) {

						var sid = sids[ s ];
						var next = animation.getNextKeyWith( sid, h, 0 );

						if ( next ) next.apply( sid );

					}

					obj.matrixAutoUpdate = false;
					animation.data.hierarchy[ h ].node.updateMatrix();
					obj.matrixWorldNeedsUpdate = true;

				}

			}

			animation.play( false, 0 );
			this.lastTimestamp = Date.now();

			this.progress = 0;
		}


	};

	this.playKFAnimation = function (timestamp) {

		var frameTime = ( timestamp - this.lastTimestamp ) * this.animProperties.animSpeed; // seconds

		if ( this.progress >= 0 && this.progress < this.animProperties.animDuration ) {

			for ( var i = 0; i < this.kfAnimationsLength; ++i ) {

				this.kfAnimations[ i ].update( frameTime );

			}

		} else if ( this.progress >= this.animProperties.animDuration ) {

			for ( var i = 0; i < this.kfAnimationsLength; ++i ) {

				this.kfAnimations[ i ].stop();

			}

			if (this.animProperties.loop == false) {
				this.isPlaying = false;
			} else {
				this.restartKFAnimation ();
			}

			this.progress = 0;

		}

		//console.log ("here too. progress: " + this.progress + " frameTime: " + frameTime + " timestamp: " + timestamp);

		if (frameTime < 1.0) {
			this.progress += frameTime;
		} else {
			this.progress = 0;
		}
		this.lastTimestamp = timestamp;


	};

	/*
	   Updates the current state of the component - plays any running animations
	   @param {...Number} gameTime - the current game time
	 */
	this.update = function() {
		if (!this.isPlaying) {
			return;
		}

		gameTime = ENGINE.GameTime.timeSinceSceneLoad ();


		if (this.animType == "MORF") {

			for ( var i = 0; i < this.skin.morphTargetInfluences.length; i++ ) {
				this.skin.morphTargetInfluences[i] = 0;
			}

			this.weight = this.weight + this.WEIGHT_INC;
			this.skin.morphTargetInfluences [this.currSkin] = 1 - this.weight;
			this.skin.morphTargetInfluences [this.nextSkin] = this.weight;
			if (this.weight >= 1.0) {
				// advance to next pair of frames (morph targets)
				this.weight = 0.0;
				this.skin.morphTargetInfluences [this.currSkin] = 0.0;
				this.skin.morphTargetInfluences [this.nextSkin] = 1.0;

				this.currSkin = this.currSkin + 1;
				this.nextSkin = this.nextSkin + 1;

				// loop around
				if (this.currSkin >= this.animProperties.END_SKIN) {
					this.currSkin = this.animProperties.BEGIN_SKIN;
				}
				if (this.nextSkin >= this.animProperties.END_SKIN) {
					this.nextSkin = this.animProperties.BEGIN_SKIN;
				}
			}

		} else if (this.animType == "KEYF") {
			this.playKFAnimation (gameTime);

		}


		return;
	};

	/*
	   Removes this component
	 */
	this.destroy = function() {

	};


	/*
	   Set the current animation
	 */
	this.playAnimation = function (anim) {
		if (this.isPlaying && this.nowPlaying == anim) {
			return;
		}
		if (this.isPlaying && this.animType == "KEYF") {
			return;
		}

		if (anim == "swim_cycle") {
			this.animProperties = {
				"BEGIN_SKIN":2,
				"END_SKIN":4
			};

			this.currSkin = this.animProperties.BEGIN_SKIN;
			this.nextSkin = this.currSkin + 1;

			this.isPlaying = true;
		} else if (anim == "jump_cycle") {
			this.animProperties = {
				"BEGIN_SKIN":5,
				"END_SKIN":10
			};

			this.currSkin = this.animProperties.BEGIN_SKIN;
			this.nextSkin = this.currSkin + 1;

			this.isPlaying = true;
		}

		if (anim == "anim") {
			this.animType = "KEYF";
			this.isPlaying = true;
			this.progress = 0;
			this.animProperties = {
				"loop":true,
				"animDuration":2.0,
				"animSpeed":1.0
			};
			this.restartKFAnimation ();
			console.log ("Starting anim: " + this.nowPlaying);
		}

		this.nowPlaying = anim;
	}

	/*
	   Resume paused animation
	 */
	this.resumeAnimation = function () {
		this.isPlaying = true;
	}

	/*
	   Pause the current animation
	 */
	this.pauseAnimation = function () {
		this.isPlaying = false;
	}

	/*
	   Stop the current animation
	 */
	this.stopAnimation = function () {
		this.isPlaying = false;

		if (this.animType == "MORF") {
			for ( var i = 0; i < this.skin.morphTargetInfluences.length; i++ ) {
				this.skin.morphTargetInfluences[i] = 0;
			}
		}
	}


};
