//audioPlayer.js
//Component that allows for easy playing of sounds from associated game objects
//@author:Preston Johnson

//Sets up the player component
// ALL PARAMS ARE REQUIRED
//PARAM:validSoundCues - a 2D array in sepcified order with enum indexes for the valid sounds for this gObj
//EX validSoundCues[0] is an array of attack sounds. validSoundCues[0][i] is the enum (see audio.js/LevelSoundEnums) of the sound to play
//   validSoundCues[1] is an array of hit sounds , validSoundCues[1][i] " " 
//   validSoundCues[2] is an array of voices
//	 validSoundCues[3] is an array of move sounds //NOT IMPLMEENTED IN GALUNTLET - NO MOVEMENT SOUNDS
//   validSoundCues[4] is an array of misc sounds
function AudioPlayer(validSoundCues,gameObj)
{
	this.type = 'AudioPlayer'; // Definition Required by all component objects
	this.ready = false;

	if(Audio==null||gameObj==null||validSoundCues==null)
	{
		//if the audio engine doesn't exist or we are missing data then stop processing
		return null;
	}
	
	var gObj = gameObj;

	// plays sound
	this.play = function(soundType,cueNum)
	{
		if(validSoundCues[soundType] == null)
		{
			//No sounds of sepcified type for this object, return
			return;
		}
		
		//if specific cue is given just play it
		if(cueNum!=null)
		{
			Audio.PlaySound(cueNum,gObj);
			return;
		}
		
		//no cue specified, play a random sound of the type
		if(cueNum == null)
		{
			//Determine random cue
			cueNum = Math.floor(Math.random()*validSoundCues[soundType].length);
		}
		else if(cueNum < 0)
		{
			//Determine random cue
			cueNum = Math.floor(Math.random()*validSoundCues[soundType].length);
		}
		
		//Play the sound
		Audio.PlaySound(validSoundCues[soundType][cueNum],gObj);
		
	}
	
	
	//Event listener, plays a random hit sound on hit
	//PARAM: the specific cue of the sound to play, null will play a random sound of the specified type
	this.onhit = function(cue)
	{
		this.play(1,cue);
	}
	
	
	//NOT IMPLMEENTED IN GALUNTLET - NO MOVEMENT SOUNDS
	//Event listener, plays the movement sound
	//PARAM: the specific cue of the sound to play, null will play a random sound of the specified type
	this.onmove = function(cue)
	{
		this.play(3,cue);
	}
	
	//Event listener, plays misc sound
	//PARAM (REQUIRED): the specific cue of the sound to play, null will do nothing
	this.onmisc = function(cue)
	{
		if(cue!=null)
		{
			this.play(4,cue);
		}
	}
	
	//Event listener, plays voice cue
	//PARAM (REQUIRED): the specific cue of the sound to play, null will do nothing
	this.onvoice = function(cue)
	{
		if(cue!=null)
		{
			this.play(2,cue);
		}
	}
	
	//Event listener, plays attack sound
	//PARAM: the specific cue of the sound to play, null will play a random sound of the specified type
	this.onattack = function(cue)
	{
		this.play(0,cue);
	}
	
	this.ready = true;
};
