// Component for CameraTracker
// Mark Thoughts - Update is called when a player moves, and it tries to reset the camera accordingly.
// If the players movement causes the camera to move such that another player is now offscreen, move that player back
// TODO: Currently it assumes greater than 1 player, may need change how camera works with only one player

var CameraTracker = function () {
	//How far the camera is behind the center point (using these values we get 30-60-90 triangles
	this.TRAIL_DISTANCE_X = 0.0;
	this.TRAIL_DISTANCE_Y = 8.66025;
	this.TRAIL_DISTANCE_Z = 5.0;
	
	//The minimum trail distance for the camera
	this.MIN_TRAIL_DISTANCE_Y = 7.0;
	this.MIN_TRAIL_DISTANCE_Z = 7.0;
	
	//The maximum trail distance for the camera
	this.MAX_TRAIL_DISTANCE_Y = 7.0;
	this.MAX_TRAIL_DISTANCE_Z = 7.0;
	
	//The values for how far away the sides are from center point
	this.bottom_left = new THREE.Vector3( 0, 0, 0 );
	this.bottom_right = new THREE.Vector3( 0, 0, 0 );
	this.top_left = new THREE.Vector3( 0, 0, 0 );
	this.top_right = new THREE.Vector3( 0, 0, 0 );
	
	//center of where the camera is looking
	this.center = new THREE.Vector3( 0, 0, 0 );
	
	//THIS ANGLE IS ASSUMING A FOV OF 60 AND A 16/9 ASPECT RATIO **************Z May need to be adjusted due to UI************
	this.CAMERA_X_ANGLE = 30;
	this.CAMERA_Z_ANGLE = 16.875;
	
	this.lookatTargets = new Array();
	this.objectsOnScreen = new Array();
	this.predictivePositions = new Array();
	
	this.playerOffScreen = false;
	var tempVel = new THREE.Vector3(0,0,0);

	this.type = "CAMERTRACKER_CP";			// CHANGE THIS !

	/*
	   Initializes the CameraTracker component to be added to a Camera object
	 */
	this.init = function(){
		this.lookatTargets = new Array();
		this.predictivePositions = new Array();
		this.objectsOnScreen = new Array();

	};

	/*
	   Updates the current state of the component - makes the camera track the target
	   @param {...Number} gameTime - the current game time
	 */
	this.superEarlyUpdate = function(gameTime)
	{
		this.predictivePositions = new Array();
		if (this.lookatTargets[0] != undefined)
		{
			for (var i=0; i<this.lookatTargets.length; i++)
			{
				this.tempVel = new THREE.Vector3(this.lookatTargets[i].getComponent('RigidBody').velocity.x,0,this.lookatTargets[i].getComponent('RigidBody').velocity.z);
				this.predictivePositions.push(new THREE.Vector3(this.lookatTargets[i].position.x, 0 , this.lookatTargets[i].position.z));
				//console.log("Before: ");
				//console.log("" + this.lookatTargets[i].position.x + " " + this.lookatTargets[i].position.y + " " + this.lookatTargets[i].position.z);
				this.predictivePositions[i] = this.predictivePositions[i].add(this.tempVel.multiplyScalar(gameTime.deltaTime()));
				//console.log("After: ");
				//console.log("" + this.predictivePositions[i].x + " " + this.predictivePositions[i].y + " " + this.predictivePositions[i].z);
			}
			//console.log(this.lookatTargets[0].position);
			//console.log(this.predictivePositions[0]);
			var left, right, near, far;
			var leftIndex, rightIndex, nearIndex, farIndex;
			var centrex, centrez;
			left = this.predictivePositions[0].x;
			right = this.predictivePositions[0].x;
			near = this.predictivePositions[0].z;
			far = this.predictivePositions[0].z;
			leftIndex = 0;
			rightIndex = 0;
			nearIndex = 0;
			farIndex = 0;
			centrex = left;
			centrez = near;
			if ( this.lookatTargets[1] != undefined)
			{
				for (var i=1; i<this.predictivePositions.length; i++)
				{
					if (this.predictivePositions[i].x < left)
					{					
						left  = this.predictivePositions[i].x;
						leftIndex = i;
					}
					if (this.predictivePositions[i].x > right)		
					{
						right = this.predictivePositions[i].x;
						rightIndex = i;
					}
					if (this.predictivePositions[i].z > near)		
					{
						near  = this.predictivePositions[i].z;
						nearIndex = i;
					}
					if (this.predictivePositions[i].z < far)		
					{
						far  = this.predictivePositions[i].z;
						farIndex = i;
					}
				}
			}
			centrex = (left + right) / 2;
			centrez = (near + far) / 2;
			
			center = new THREE.Vector3(centrex, 0.0, centrez);

			this.gameObject.position.x = center.x + this.TRAIL_DISTANCE_X;
			this.gameObject.position.y = center.y + this.TRAIL_DISTANCE_Y;
			this.gameObject.position.z = center.z + this.TRAIL_DISTANCE_Z;
			
			this.RecalculateBounds();
			if (near > this.bottom_right.z)
			{
				//TODO: After initial if, check to see if both objects are going the same direction, if they are, calculate how far they are allowed to move
				if (this.lookatTargets[farIndex].getComponent('RigidBody').velocity.z < 0)
					this.lookatTargets[farIndex].getComponent('RigidBody').velocity.z = .00001;
				if (this.lookatTargets[nearIndex].getComponent('RigidBody').velocity.z > 0)
					this.lookatTargets[nearIndex].getComponent('RigidBody').velocity.z = .00001;
			}
			
			var slope = (this.top_right.z - this.bottom_right.z) / (this.top_right.x - this.bottom_right.x);
			var xToCheckR = (this.lookatTargets[rightIndex].position.z - this.bottom_right.z) / slope;
			var xToCheckL = (this.lookatTargets[leftIndex].position.z - this.bottom_right.z) / slope;
			
			if (right > (this.bottom_right.x + xToCheckR) || left < (this.bottom_left.x - xToCheckL))
			{
				if (this.lookatTargets[rightIndex].getComponent('RigidBody').velocity.x > 0)
					this.lookatTargets[rightIndex].getComponent('RigidBody').velocity.x = .00001;
				if (this.lookatTargets[leftIndex].getComponent('RigidBody').velocity.x < 0)
					this.lookatTargets[leftIndex].getComponent('RigidBody').velocity.x = .00001;
			}
			
		}
	};
	 
	this.update = function(gameTime) {
		// get required bounds:
		if(this.lookatTargets[0] != undefined)
		{
		
			var left, right, near, far;
			var centrex, centrez;
			left = this.lookatTargets[0].position.x;
			right = this.lookatTargets[0].position.x;
			near = this.lookatTargets[0].position.z;
			far = this.lookatTargets[0].position.z;
			centrex = left;
			centrez = near;
			if ( this.lookatTargets[1] != undefined)
			{
				for (var i=1; i<this.lookatTargets.length; i++) {
					if (this.lookatTargets[i].position.x < left)		left  = this.lookatTargets[i].position.x;
					if (this.lookatTargets[i].position.x > right)		right = this.lookatTargets[i].position.x;
					if (this.lookatTargets[i].position.z > near)		near  = this.lookatTargets[i].position.z;
					if (this.lookatTargets[i].position.z < far)		far  = this.lookatTargets[i].position.z;
					//centrex += this.lookatTargets[i].position.x;
					//centrez += this.lookatTargets[i].position.z;
				}
			}
			centrex = (left + right) / 2;
			centrez = (near + far) / 2;
			
			//verify this is a good Y value
			center = new THREE.Vector3(centrex, 0.0, centrez);

			this.gameObject.position.x = center.x + this.TRAIL_DISTANCE_X;
			this.gameObject.position.y = center.y + this.TRAIL_DISTANCE_Y;
			this.gameObject.position.z = center.z + this.TRAIL_DISTANCE_Z;
			
			this.RecalculateBounds();
			/*
			this.shadowLight.position.x = this.gameObject.position.x + 10.0;
			this.shadowLight.position.y = this.gameObject.position.y;
			this.shadowLight.position.z = this.gameObject.position.z;

			*/
			this.gameObject.lookAt (center);
			
		}
	};

	/*
	   Removes this component
	 */
	this.destroy = function() {
		
	};
	
	/*
	   Calculate the corners of the screen
	 */
	this.RecalculateBounds = function() {
		//the X side is symmetrical, though there is a possibility of error.
		var center_bottom = new THREE.Vector3(center.x,0,center.z + (Math.sqrt((this.TRAIL_DISTANCE_Y * this.TRAIL_DISTANCE_Y) + (this.TRAIL_DISTANCE_Z *this.TRAIL_DISTANCE_Z)) * (Math.sin(16.875 * (Math.PI / 180)) / Math.sin(103.125 * (Math.PI / 180)))));
		var bottom_x_offset = (Math.sqrt((this.TRAIL_DISTANCE_Y * this.TRAIL_DISTANCE_Y) + ((this.TRAIL_DISTANCE_Z - (center_bottom.z - center.z)) * (this.TRAIL_DISTANCE_Z - (center_bottom.z - center.z)))))/Math.sqrt(3);
		
		this.bottom_left = new THREE.Vector3(center_bottom.x - bottom_x_offset, 0, center_bottom.z + 2);
		this.bottom_right = new THREE.Vector3(center_bottom.x + bottom_x_offset, 0, center_bottom.z + 2);
		
		var center_top = new THREE.Vector3(center.x,0,center.z - (Math.sqrt((this.TRAIL_DISTANCE_Y * this.TRAIL_DISTANCE_Y) + (this.TRAIL_DISTANCE_Z *this.TRAIL_DISTANCE_Z)) * (Math.sin(16.875 * (Math.PI / 180)) / Math.sin(43.125 * (Math.PI / 180)))));
		var top_x_offset = 4 + (Math.sqrt((this.TRAIL_DISTANCE_Y * this.TRAIL_DISTANCE_Y) + ((this.TRAIL_DISTANCE_Z + (center.z - center_top.z)) * (this.TRAIL_DISTANCE_Z + (center.z - center_top.z)))))/Math.sqrt(3);
		
		this.top_left = new THREE.Vector3(center_top.x - top_x_offset, 0, center_top.z - 4);
		this.top_right = new THREE.Vector3(center_top.x + top_x_offset, 0, center_top.z - 4);
	};
	
	/*
		Try and move the camera in or out if necessary. 
	*/
	this.CalculateDistance = function(){
		//loop through all objects we are looking at and get the furthest right, left, and near
		//make sure we have 2 characters
		/*if(this.lookatTargets[0] != undefined)
		{
			var near = this.lookatTargets[0].position.z;
			var right = this.lookatTargets[0].position;
			var left = this.lookatTargets[0].position;
			if ( this.lookatTargets[1] != undefined)
			{
				for (var i=0; i<this.lookatTargets.length; i++)
				{
					if (this.lookatTargets[i].position.z > near)
						near = this.lookatTargets[i].position.z;
					if (this.lookatTargets[i].position.x > right)
						right = this.lookatTargets[i].position;
					if (this.lookatTargets[i].position.x < left)
						left = this.lookatTargets[i].position;
				}
			}
			var distanceNearRequired = (near - center.z) * ((Math.sin(103.125 * (Math.PI / 180)))/(Math.sin(16.875 * (Math.PI / 180))));
			var greaterX;
			if (Math.abs(left.x - center.x) > Math.abs(right.x - center.x))
				greaterX = left;
			else
				greaterX = right;
			
			var distanceXRequired = (greaterX.z - center.z) * ((Math.sin(103.125 * (Math.PI / 180)))/(Math.sin(16.875 * (Math.PI / 180))));
		}*/
	};
	
	
	/*
	   Given a Vector3, is that position on screen. Checks x,z coordinates against the camera's FOV
	 */
	this.IsOnScreen = function(target) {
		try
		{
			var alreadyIn = -1;
			if(this.objectsOnScreen[0] != undefined)
			{
				for (var i=0;i<this.objectsOnScreen.length;i++)
				{ 
					if(this.objectsOnScreen[i] == target)
						alreadyIn = i;
						//console.log(i+" "+target.getComponent("playerChar").type); Only thing in the list is players
						
				}
			}
			//check Z values first, simplicity
			//console.log("TargetZ: " + target.position.z + ", BottomZ: " + this.bottom_right.z + ", TopZ: " + this.top_right.z );
			if (target.position.z < this.bottom_right.z)
			{
				if (target.position.z > this.top_right.z)
				{
					//console.log("Passed Z Test");
					var slope = (this.top_right.z - this.bottom_right.z) / (this.top_right.x - this.bottom_right.x);
					var xToCheck = (target.position.z - this.bottom_right.z) / slope;
					//console.log("Checking X");
					//console.log("TargetX: " + target.position.x + ", RightX: " + (this.bottom_right.x + xToCheck) + ", LeftX: " + (this.bottom_left.x - xToCheck) + ", xToCheck: " + xToCheck);
					if (target.position.x < (this.bottom_right.x + xToCheck))
					{
						if (target.position.x > (this.bottom_left.x - xToCheck))
						{
							//console.log(this.objectsOnScreen);
							if (alreadyIn == -1)
								this.objectsOnScreen.push(target);
							return true;
						}

					}
				}
			}
			if (alreadyIn >= 0)
				this.objectsOnScreen.splice(alreadyIn,1);
			//console.log(this.objectsOnScreen);
		//	console.log(this.objectsOnScreen);
			return false;
		}
		catch (err)
		{
			console.log(err);
		}
	};

	/*
	   Set the lookAt target for the camera to track
	 */
	this.addLookAt = function (target) {
		this.lookatTargets.push (target);
		this.predictivePositions.push(new THREE.Vector3(0,0,0));
	};
	
	this.removeLookAt = function(target) {
		this.lookatTargets.splice(this.lookatTargets.indexOf(target),1);
	};

	this.setTrailDistance = function (X, Y, Z) {
		this.TRAIL_DISTANCE_X = X;
		this.TRAIL_DISTANCE_Y = Y;
		this.TRAIL_DISTANCE_Z = Z;
	};

};
