//Collision Manager

/* 
	 *	The collision manager component.  There should only be one per scene.
	 *	@param {int} w the width (in pixels) of the 2D map.
	 *	@param {int} h the height (in pixels) of the 2D map.
	 */
var collisionManager = function(w, h) {
	
	this.type = "collisionManager";
	this.bins = new Array();
	this.collidables = new Array();
	
	this.minimalSpacing = 0.1;
	
	if(w == null)
		this.mapWidth = 100;
	else
		this.mapWidth = w;
		
	if(h == null)
		this.mapHeight = 100;
	else
		this.mapHeight = h;
	
	this.binSize = 10;
	this.binsWidth = Math.ceil(this.mapWidth/this.binSize);
	this.binsHeight = Math.ceil(this.mapHeight/this.binSize);
	
	//debugging bins:
	this.green = new Array();
	
	this.init = function(){
	
		for(var i = 0; i < this.mapHeight; i += this.binSize)
		{
			for(var j = 0; j < this.mapWidth; j += this.binSize)
			{
				var b = new Object();
				b.size = this.binSize;
				b.x = j;// + (this.mapWidth/2);
				b.z = i;// + (this.mapHeight/2);
				b.y = 0;
				b.pos = (j/this.binSize) + ", " + (i/this.binSize) ;
				b.containedObjs = new Array();
				this.bins.push(b);
			}
		}
	};
	
	this.earlyUpdate = function(){
		//reset debug bins
		/*var c = ENGINE.currentScene.scene.getChildByName('bin4');
		if(c)
		{
			console.log(c);
			ENGINE.currentScene.remove(c);
		}
		/**************OLD***************************/
		
		/*for(j = 0; j < this.green.length; j++)
		{
			if(ENGINE.currentScene.scene.getChildByName('bin' + this.green[j]) != undefined)
			{
				var c = ENGINE.currentScene.scene.getChildByName('bin' + this.green[j]);
				c.material = new THREE.ShaderMaterial( { vertexColors: 0xffffff, wireframe: true });
			}
		}*/
		
		var oldPositions = new Array();
		
		// Set Velocities from rigidbody data
		for(var w = 0; w < this.collidables.length; w++)
		{
			var collider = this.collidables[w];
			var gameObject = collider.gameObject;
			var rigidbody = gameObject.getComponent('RigidBody');
			
			// Set velocity according to where the position should be
			if(rigidbody._setPosition)
			{
				rigidbody.velocity = rigidbody._setPosition.clone().sub(gameObject.position).divideScalar(ENGINE.GameTime.deltaTime());
			}
			// Apply Force
			else
			{
				rigidbody.velocity.add(rigidbody._netForce.clone().multiplyScalar(ENGINE.GameTime.deltaTime()/rigidbody.mass));
			}
			// Set angular velocity according to where the rotation should be
			if(rigidbody._setRotation)
			{
				rigidbody.angularVelocity = rigidbody._setRotation.clone().sub(gameObject.rotation).divideScalar(ENGINE.GameTime.deltaTime());
			}
			else
			{
				rigidbody.angularVelocity.add(rigidbody._netTorque.clone().divide(rigidbody.inertia).multiplyScalar(ENGINE.GameTime.deltaTime()));
			}
			
			// Reset
			rigidbody._setPosition = null;
			rigidbody._setRotation = null;
			
			rigidbody._netForce = new THREE.Vector3(0,0,0);
			rigidbody._netTorque = new THREE.Vector3(0,0,0);
			
			// store the old position
			oldPositions[w] = gameObject.position.clone();
		}

		this.green = new Array();
		var fullBins = this.getBins();
		//clear the contacts of objects
		for(i = 0; i < this.collidables.length; i++)
		{
			var obj = this.collidables[i];
			obj.clearContacts();
		}
		
		for(j = 0; j < fullBins.length; j++)
		{
			var b = fullBins[j];
			var contacts = new Array();
			
			for(i = 0; i < this.collidables.length; i++)
			{
				var obj = this.collidables[i];
					
				if(this.isInBin(b,obj))
				{	
					//if(obj.gameObject.tag == "wall")
						//console.log("wall in bin " + b.pos);
					contacts.push(obj);
				}
			}
			// Getting rid of optimization here. TODO: FIX
			for(var a = 0; a < /*this.collidables.*/contacts.length; a++)
			{
				for(var b = 0; b < /*this.collidables.*/contacts.length; b++)
				{
					if (a == b)
						continue;
						
					var colliderA = /*this.collidables*/contacts[a];
					var colliderB = /*this.collidables*/contacts[b];
					
					if(!checkIgnores(colliderA,colliderB))
					{
						var contact = null;
						if(colliderA.boundType == "rectangle" && colliderB.boundType == "rectangle")
							//contacts = this.AABBSweep(colliderA, colliderB);
							contact = this.AABBCollider(colliderA, colliderB);
						
						if(contact)
						{
							colliderA.contacts.push(contact/*s.contactA*/);
							//colliderB.contacts.push(contacts.contactB);
							colliderA.colliding = true;
							//colliderB.colliding = true;
							
							// MOVE IMMEDIATELY?!
							// Stop moving in the direction of collision
							
							
							// Only is not ignoring what it collided with
							if(!checkIgnoreResponse(colliderA, colliderB))
							{
								var amountProjected = colliderA.gameObject.getComponent('RigidBody').velocity.dot(contact.normal); // Add spacing?
								if (amountProjected > 0)
								{
									colliderA.gameObject.getComponent('RigidBody').velocity.sub(contact.normal.clone().multiplyScalar(amountProjected));
								}
							}
							else
							{
								//console.log("HAI");
							}
							// ONLY IF I AM A MOVING OBJECT DO YOU HAVE PERMISSION TO MOVE ME
							/*if(colliderA.gameObject.getComponent('RigidBody').velocity.lengthSq() > 0)
							{
								colliderA.gameObject.position = contact.point;
							}*/
						}
					}
				}
			}
		}
		for(w = 0; w < this.collidables.length; w++)
		{
			var collider = this.collidables[w];
			var gameObject = collider.gameObject;
			var rigidbody = gameObject.getComponent('RigidBody');
			gameObject.position = oldPositions[w];
			gameObject.position.add(rigidbody.velocity.clone().multiplyScalar(ENGINE.GameTime.deltaTime()));
			// Apply angularVelocity.
			gameObject.rotation.add(rigidbody.angularVelocity.clone().multiplyScalar(ENGINE.GameTime.deltaTime()));
		}
		
		//change bins to green
		for(j = 0; j < this.green.length; j++)
		{
			if(ENGINE.currentScene.scene.getChildByName('bin' + this.green[j]) != undefined)
			{
				var c = ENGINE.currentScene.scene.getChildByName('bin' + this.green[j]);
				c.material = new THREE.MeshBasicMaterial( { color: 0xff0000, wireframe: true });
			}
		}
	};

	/*
		Represents a contact point
	*/
	var Contact = function(point, normal, relativeVelocity, timeOfCollision, thisCollider, otherCollider)
	{
		this.point = point;  // Where the gameObject was when the collision happened
		this.normal = normal;	// The normal of the collision
		this.relativeVelocity = relativeVelocity; // A with respect to B?
		this.timeOfCollision = timeOfCollision; // In NORMALIZED time, 0-1 for frame
		this.thisCollider = thisCollider;	// Reference to this object's collider
		this.otherCollider = otherCollider;	// Reference to the other object's collider
	};
	

	/* returns a list of bins that contain objects */
	this.getBins = function()
	{
		var fullBins = new Array();
		
		var toCheck = new Array();
		for(var i = 0; i < this.collidables.length; i++)
		{
			var collider = this.collidables[i];

			if(collider.gameObject.getComponent("playerChar") || collider.gameObject.getComponent("enemyChar") || collider.gameObject.getComponent("projectile") || collider.gameObject.tag == "wall")
			{
				toCheck.push(collider);
			}
			
		}
		
		
		for(var i = 0; i < toCheck.length; i++)
		{
			var collider = toCheck[i];
			//collider.debugDraw();
			//top left
			var xPos = collider.gameObject.position.x;
			var zPos = collider.gameObject.position.z;
			
			
			var col = Math.round(xPos/this.binSize);
			
			var row = Math.round(zPos/this.binSize);
			
			var index = row * this.binsHeight + col;
			index = parseInt(index);
			
			//console.log("bin " + index + "\n xPos " + xPos + "\n zPos " + zPos + "\n row " + row + "\n col " + col);
			
			
			if(this.bins[index] && this.green.indexOf(index) == -1)
			{
				//console.log("pushing bin " + index + "\n xPos " + xPos + "\n zPos " + zPos);
				fullBins.push(this.bins[index]);
				this.green.push(index);
				
				col--;
				index = row * this.binsHeight + col;
				
				
			}
			
			/*
			
			//top right
			xPos = collider.gameObject.position.x + (collider.boundWidth || collider.boundRadius);

			col = (Math.round( ((xPos/this.binSize))) * this.binsWidth ) / this.binsWidth ;

			index = row * this.binsHeight + col;
			index = parseInt(index);

			if(this.bins[index] && this.green.indexOf(index) == -1)
			{
				//console.log("pushing bin " + index + "\n xPos " + xPos + "\n zPos " + zPos);
				fullBins.push(this.bins[index]);
				this.green.push(index);
			}
			
			
			
			//bottom left
			zPos = collider.gameObject.position.z + (collider.boundHeight || collider.boundRadius);

			row = (Math.round( ((zPos/this.binSize))) * this.binsHeight ) / this.binsHeight ;

			index = row * this.binsHeight + col;
			index = parseInt(index);

			if(this.bins[index] && this.green.indexOf(index) == -1)
			{
				//console.log("pushing bin " + index + "\n xPos " + xPos + "\n zPos " + zPos);
				fullBins.push(this.bins[index]);
				this.green.push(index);
			}
			
			
			//bottom right
			xPos = collider.gameObject.position.x + (collider.boundWidth || collider.boundRadius);

			col = (Math.round( ((xPos/this.binSize))) * this.binsWidth ) / this.binsWidth ;

			index = row * this.binsHeight + col;
			index = parseInt(index);

			if(this.bins[index] && this.green.indexOf(index) == -1)
			{
				//console.log("pushing bin " + index + "\n xPos " + xPos + "\n zPos " + zPos);
				fullBins.push(this.bins[index]);
				this.green.push(index);
			}
			
			*/
		}
		//console.log(fullBins);
		return fullBins;
	};
	

	/*
		Detects a collision between 2 rectangle bounds
		@param {collider} A The collider component of the first object
		@param {collider} B The collider component of the second object
	*/
	this.AABBCollider = function(A, B)
	{		
		var aw;		// Bound Width of A
		var bw;	 	// Bound Width of B
		var ah;		// Bound Height of A	
		var bh;		// Bound height of B
		var ad;		// Bound Depth of A
		var bd;		// Bound Depth of B
		
		// Set bounds d on collider properties
		if(A.boundWidth && A.boundHeight)
		{
			aw = A.boundWidth;
			ah = A.boundHeight;
			ad = A.boundDepth || A.boundHeight;
		}
		else
		{
			aw = A.gameObject.boundRadius;
			ah = A.gameObject.boundRadius;
			ad = A.gameObject.boundRadius;
		}
		
		if(B.boundWidth && B.boundHeight)
		{
			bw = B.boundWidth;
			bh = B.boundHeight;
			bd = B.boundDepth || B.boundHeight;
		}
		else
		{
			bw = B.gameObject.boundRadius;
			bh = B.gameObject.boundRadius;
			bd = B.gameObject.boundRadius;
		}
		
		// Dom's method to fake out normal
		var aMinX = A.gameObject.position.x - aw/2;
		var aMaxX = A.gameObject.position.x + aw/2;
		var bMinX = B.gameObject.position.x - bw/2;
		var bMaxX = B.gameObject.position.x + bw/2;

		// Y axis
		var aMinY = A.gameObject.position.y - ah/2;
		var aMaxY = A.gameObject.position.y + ah/2;
		var bMinY = B.gameObject.position.y - bh/2;
		var bMaxY = B.gameObject.position.y + bh/2;
		
		// Z axis
		var aMinZ = A.gameObject.position.z - ad/2;
		var aMaxZ = A.gameObject.position.z + ad/2;
		var bMinZ = B.gameObject.position.z - bd/2;
		var bMaxZ = B.gameObject.position.z + bd/2;
		
		var point = new THREE.Vector3(0,0,0);
		
		var overlapX = 0;
		var xNormal = 1;
		if (aMinX < bMinX)
		{
			overlapX = aMaxX - bMinX;
			point.x = bMinX - aw/2;
		}
		else
		{
			overlapX = bMaxX - aMinX;
			xNormal = -1;
			point.x = bMaxX + aw/2;
		}

		var overlapY = 0;
		var yNormal = 1;
		if (aMinY < bMinY)
		{
			overlapY = aMaxY - bMinY;
			point.y = bMinY - ah/2;
		}
		else
		{
			overlapY = bMaxY - aMinY;
			yNormal = -1;
			point.y = bMaxY + ah/2;
		}
		
		var overlapZ = 0;
		var zNormal = 1;
		if (aMinZ < bMinZ)
		{
			overlapZ = aMaxZ - bMinZ;
			point.z = bMinZ - ad/2;
		}
		else
		{
			overlapZ = bMaxZ - aMinZ;
			zNormal = -1;
			point.z = bMaxZ + ad/2;
		}
		
		// We are overlapping along all axes.  Must be a collision.
		if(overlapX > 0 && overlapY > 0 && overlapZ > 0 )
		{
			var normal = new THREE.Vector3(0,0,0);
			
			// X is the axis of least overlap
			if(overlapX <= overlapY && overlapX <= overlapZ)
			{
				normal = new THREE.Vector3(xNormal, 0,0);
				//overlapVector.x = xNormal*overlapX;
				point.y = A.gameObject.position.y; // reset y
				point.z = A.gameObject.position.z; // reset z
			}
			// Y is the axis of least overlap
			else if(overlapY <= overlapX && overlapY <= overlapZ)
			{
				normal = new THREE.Vector3(0,yNormal,0);
				//overlapVector.y = yNormal*overlapY;
				point.x = A.gameObject.position.x; // reset x
				point.z = A.gameObject.position.z; // reset z
			}
			// Z is the axis of least overlap
			else /*if(overlapZ <= overlapX && overlapZ <= overlapY)*/
			{
				normal = new THREE.Vector3(0,0,zNormal);
				//overlapVector.z = zNormal*overlapZ;
				point.x = A.gameObject.position.x; // reset x
				point.y = A.gameObject.position.y; // reset y
			}
			
			var va = A.gameObject.getComponent('RigidBody').velocity.clone();
			var vb = B.gameObject.getComponent('RigidBody').velocity.clone();
			var vab = vb.clone().sub(va);
			
			// return a contact point
			//(point, normal, relativeVelocity, timeOfCollision, thisCollider, otherCollider)
			return new Contact(point, normal, vab, 0, A, B);
		}
		return null;
		
		/* Ryan's Old Mehtod
		// Calculate distances from gameObject positions
		var distX = B.gameObject.position.x - A.gameObject.position.x;
		var distY = B.gameObject.position.y - A.gameObject.position.y;
		var distZ = B.gameObject.position.z - A.gameObject.position.z;
		
		if(distX < 0)
			distX *= -1;
		if(distY < 0)
			distZ *= -1;
		if(distZ < 0)
			distZ *= -1;
		
		// Add abs y here?
		
		var r = (aw) + (bw);
		
		xOverlap = (distX <= (r));
		
		r = (ah) + (bh);
		
		zOverlap = (distZ <= (r));
		
		r = (ad) + (bd);
		
		yOverlap = (distY <= (r));
					
		return xOverlap && zOverlap && yOverlap;*/
	}
	
	this.valueInRange = function(value, min, max)
	{ return (value >= min) && (value <= max); }
	
	/*
		checks if the objects should ignore each others collision. i.e., food and enemies
	*/
	var checkIgnores = function(obj1, obj2)
	{
		if(obj1.ignores)
		{
			for(var i = 0; i < obj1.ignores.length; i++)
			{
				if(obj2.gameObject.getComponent(obj1.ignores[i]))
				{
					return true;
				}
			}
		}
		
		if(obj2.ignores)
		{
			for(var i = 0; i < obj2.ignores.length; i++)
			{
				if(obj1.gameObject.getComponent(obj2.ignores[i]))
				{
					return true;
				}
			}
		}
		
		if(obj1.gameObject.tag == "wall" && obj2.gameObject.tag == "wall")
			return true;
		
		return false;
		
	};
	
	/*
		checks if the objects should ignore each others collision. i.e., food and enemies
	*/
	var checkIgnoreResponse = function(obj1, obj2)
	{
		if(obj1.ignoreResponse)
		{
			for(var i = 0; i < obj1.ignoreResponse.length; i++)
			{
				if(obj2.gameObject.getComponent(obj1.ignoreResponse[i]))
				{
					return true;
				}
			}
		}
		
		if(obj2.ignoreResponse)
		{
			for(var i = 0; i < obj2.ignoreResponse.length; i++)
			{
				if(obj1.gameObject.getComponent(obj2.ignoreResponse[i]))
				{
					return true;
				}
			}
		}
		return false;
	};

	
	/* 
	 *	Adds a collidable component to the list of all collidables.
	 *	@param {collider} collider the component of the gameObject to have collision detection.
	 */
	this.addCollidable = function(collider)
	{
		this.collidables.push(collider);
		collider.index = this.collidables.length - 1;
	};
	
	/* 
	 *	Deletes a collidable component from the list of all collidables.
	 *	@param {int} i the index property of the collidable component of the game object to remove.
	 */
	this.removeCollidable = function(i)
	{
		for(var j = 0; j < this.collidables.length; j++)
		{
			if(this.collidables[j].contacts.indexOf(i) != -1)
			{
				this.collidables[j].contacts.splice(this.collidables[j].contacts.indexOf(i), 1);
				if(this.collidables[j].contacts.length == 0)
					this.collidables[j].colliding = false;
			}
		}
		
		
		this.collidables.splice(i,1);
		for(var j = i; j < this.collidables.length; j++)
		{
			this.collidables[j].index--;
		}
	};
	
	
	this.isInBin = function(bin, obj)
	{
		if(obj.boundType == "rectangle")
		{
				
			/*var distX = obj.gameObject.position.x - bin.x;
			var distY = obj.gameObject.position.y - bin.y;
			var distZ = obj.gameObject.position.z - bin.z;
			
			if(distX < 0)
				distX *= -1;
			if(distZ < 0)
				distZ *= -1;
			
			var r = (bin.size/2) + (obj.boundWidth);
			
			xOverlap = (distX <= (r));
			
			r = (bin.size/2) + (obj.boundHeight);
			
			zOverlap = (distZ <= (r));
			
			r = (bin.size/2) + (obj.boundDepth);
			
			var yOverlap = (distY <= (r));
			
			
			return xOverlap && zOverlap;*/
			
			var distX = obj.gameObject.position.x - bin.x;
			var distY = obj.gameObject.position.y - bin.y;
			var distZ = obj.gameObject.position.z - bin.z;
			
			if(distX < 0)
				distX *= -1;
			if(distY < 0)
				distZ *= -1;
			if(distZ < 0)
				distZ *= -1;
			
			// Add abs y here?
			
			var r = (bin.size/2) + (obj.boundWidth/2);
			xOverlap = (distX <= (r));
			
			r = (bin.size/2) + (obj.boundHeight/2);
			yOverlap = (distY <= (r));
			
			r = (bin.size/2) + (obj.boundDepth/2);
			zOverlap = (distZ <= (r));
						
			return xOverlap && zOverlap && yOverlap;
		}
		else
		{
			var circleDistanceX = Math.abs(obj.gameObject.position.x - bin.x);
			var circleDistanceY = Math.abs(obj.gameObject.position.y - bin.y);
			var circleDistanceZ = Math.abs(obj.gameObject.position.z - bin.z);

			if (circleDistanceX > (bin.size/2 + obj.boundRadius)) { return false; }
			if (circleDistanceY > (bin.size/2 + obj.boundRadius)) { return false; }
			if (circleDistanceZ > (bin.size/2 + obj.boundRadius)) { return false; }

			if (circleDistanceX <= (bin.size/2)) { return true; } 
			if (circleDistanceY <= (bin.size/2)) { return true; }
			if (circleDistanceZ <= (bin.size/2)) { return true; }

			var cornerDistance_sq = ((circleDistanceX - bin.size/2) * (circleDistanceX - bin.size/2)) +
								 ((circleDistanceY - bin.size/2) * (circleDistanceY - bin.size/2)) +
								 ((circleDistanceZ - bin.size/2) * (circleDistanceZ - bin.size/2));

			return (cornerDistance_sq <= (obj.boundRadius * obj.boundRadius));
		}
	}
	
	
	this.destroy = function(){
		
	};
}