// Component for Animator

//idle, walking, firing, dying
var creatureAnimation = function (paused, walkTimes, fireTimes, dieTimes, speed) {

	this.walk = {};
	this.walk.start = walkTimes.start;
	this.walk.stop = walkTimes.stop;
	this.fire = {};
	this.fire.start = fireTimes.start;
	this.fire.stop = fireTimes.stop;
	this.die = {};
	this.die.start = dieTimes.start;
	this.die.stop = dieTimes.stop;
	this.pauseTime = paused;
	this.speed = speed;

	this.isPlaying = false;
	this.nowPlaying = "";

	this.weight = 0.0;
	this.WEIGHT_INC = 0.01;

	this.currSkin = 0;
	this.nextSkin = 1;

	this.animType = "MORF";		// or "KEYF"


	this.type = "creatureAnimation";			// CHANGE THIS !

	this.callback = function(){};
	this.anim = "";

	/*
	   Initializes the Animator component to be added to an Object3D
	 */
	this.init = function(){
		this.progress = 0;
	};


	/*
	   Updates the current state of the component - plays any running animations
	   @param {...Number} gameTime - the current game time
	 */
	this.update = function() {
		if (!this.isPlaying) {
			return;
		}

		gameTime = ENGINE.GameTime.timeSinceSceneLoad ();

		this.skin = this.gameObject;


		for ( var i = 0; i < this.skin.morphTargetInfluences.length; i++ ) {
			this.skin.morphTargetInfluences[i] = 0;
		}

		this.weight = this.weight + this.animProperties.animSpeed * this.WEIGHT_INC;
		this.skin.morphTargetInfluences [this.currSkin] = 1 - this.weight;
		this.skin.morphTargetInfluences [this.nextSkin] = this.weight;
		if (this.weight >= 1.0) {
			// advance to next pair of frames (morph targets)
			this.weight = 0.0;
			this.skin.morphTargetInfluences [this.currSkin] = 0.0;
			this.skin.morphTargetInfluences [this.nextSkin] = 1.0;

			this.currSkin = this.currSkin + 1 + this.animProperties.animSpeedCoarse;
			this.nextSkin = this.nextSkin + 1 + this.animProperties.animSpeedCoarse;

			// loop around
			if (this.currSkin >= this.animProperties.END_SKIN) {
				if (!this.animProperties.loop) {
					this.isPlaying = false;
				} else {
					this.currSkin = this.animProperties.BEGIN_SKIN;
				}
			}
			if (this.nextSkin >= this.animProperties.END_SKIN) {
				try {this.callback(this.anim);}catch(e){}
				if (!this.animProperties.loop) {
					this.isPlaying = false;
				} else {
					this.nextSkin = this.animProperties.BEGIN_SKIN;
				}
			}
		}


		return;
	};

	/*
	   Removes this component
	 */
	this.destroy = function() {};


	/*
	   Set the current animation
	 */
	this.playAnimation = function (anim) {
		this.anim = anim;
		if (this.isPlaying && this.nowPlaying == anim) {
			return;
		}

		if (anim == "walk") {
			this.animProperties = {
				"BEGIN_SKIN":this.walk.start,
				"END_SKIN":this.walk.stop,
				"loop":true,
				"animSpeed":this.speed ,
				"animSpeedCoarse":1
			};

			this.currSkin = this.animProperties.BEGIN_SKIN;
			this.nextSkin = this.currSkin + 1;

			this.isPlaying = true;
		} else if (anim == "fire") {
			this.animProperties = {
				"BEGIN_SKIN":this.fire.start,
				"END_SKIN":this.fire.stop,
				"loop":false,
				"animSpeed":this.speed ,
				"animSpeedCoarse":1
			};

			this.currSkin = this.animProperties.BEGIN_SKIN;
			this.nextSkin = this.currSkin + 1;

			this.isPlaying = true;
		} else if (anim == "die") {
			this.animProperties = {
				"BEGIN_SKIN":this.die.start,
				"END_SKIN":this.die.stop,
				"loop":false,
				"animSpeed":this.speed ,
				"animSpeedCoarse":1
			};

			this.currSkin = this.animProperties.BEGIN_SKIN;
			this.nextSkin = this.currSkin + 1;
		}
		this.nowPlaying = anim;
	}
	
	this.playWalk = function(callback)
	{
		this.callback = callback;
		this.playAnimation("walk");
	}
	this.playFire = function(callback)
	{
		this.callback = callback;
		this.playAnimation("fire");
	}
	this.playDie = function(callback)
	{
		this.callback = callback;
		this.playAnimation("die");
	}

	/*
	   Resume paused animation
	 */
	this.resumeAnimation = function () {
		this.isPlaying = true;
	}

	/*
	   Pause the current animation
	 */
	this.pauseAnimation = function () {
		this.isPlaying = false;
	}

	/*
	   Stop the current animation
	 */
	this.stopAnimation = function () {
		if (!this.isPlaying) {
			return;
		}

		this.isPlaying = false;

		if (this.animType == "MORF") {
			for ( var i = 0; i < this.skin.morphTargetInfluences.length; i++ ) {
				this.skin.morphTargetInfluences[i] = 0;
			}
		}
	}


};
