 // Component for player playerMovement

/*
	Constructor for the player movement component
	@param {...string} lKey - the name of the key to move left
	@param {...string} rKey - the name of the key to move right
	@param {...string} uKey - the name of the key to move up
	@param {...string} dKey - the name of the key to move down
	@param {...int} spd - the speed the object will move by
	@param {...int} input - the input object of the current Engine (ENGINE.Input)
*/
var playerMovement = function(lKey, rKey, uKey, dKey, spd){

	this.type = "playerMovement";
	this.leftKey = lKey;
	this.rightKey = rKey;
	this.upKey = uKey;
	this.downKey = dKey;
	this.speed = spd;
	
	
	/*
	Initiates the movement component to be added to an Object 3D
	*/
	this.init = function()
	{
		var light = ENGINE.LightPool.GetNewLight(LIGHT_TYPES.POINT);
		if(light!=null){
			
			light.color = new THREE.Color(0xFF5500);
			//var light = new THREE.PointLight(0xFFCCAA);
			light.position.x = 0;
			light.position.y = 4;
			light.position.z = 20;
			light.intensity = 1;
			light.distance = 10;
			light.castShadow = true;
			
			light.shadowCameraNear = ENGINE.currentScene.mainCamera.near;
			light.shadowCameraFar = ENGINE.currentScene.mainCamera.far;
			light.shadowCameraFov = ENGINE.currentScene.mainCamera.fov;
					//light.shadowCameraNear = 500;
					//light.shadowCameraFar = camera.far;
					//light.shadowCameraFov = shadowCameraFov;

			light.shadowBias = 0.005;
			light.shadowDarkness = 0.75;

			light.shadowMapWidth = 1024;//SHADOW_MAP_WIDTH;
			light.shadowMapHeight = 1024;//SHADOW_MAP_HEIGHT;
			
			light.name = this.gameObject.name + " light";
			console.log("making :"+light.name);
			console.log(light);
			light.addComponent(new Flicker(Math.random()*Math.PI*2,2,.2, 1));
			this.gameObject.add(light);
		}
	};

	/*
		Updates the current state of the component
	*/
	this.update = function(gameTime) {
		
		var xPlus = 0;
		var zPlus = 0;
		
		var moving = false;
		
		var isOnScreen = ENGINE.currentScene.mainCamera.getComponent("CAMERTRACKER_CP").IsOnScreen(this.gameObject);
		//console.log("On Screen: " + isOnScreen);
		//console.log("On Screen: " + isOnScreen);
		//console.log("On Screen: " + isOnScreen);
	//	console.log("On Screen: " + isOnScreen);
		//Check if its on screen or colliding
		
		
		// Reset velocity
		this.gameObject.getComponent('RigidBody').velocity = new THREE.Vector3(0,0,0);
		
		// Get values from ENGINE.Input, which can be a combination of booleans or floats between 1 and -1.
		var leftKeyDown = ENGINE.Input.state(this.leftKey);
		var rightKeyDown = ENGINE.Input.state(this.rightKey);
		var upKeyDown = ENGINE.Input.state(this.upKey);
		var downKeyDown = ENGINE.Input.state(this.downKey);
		// console.log(leftKeyDown, rightKeyDown, upKeyDown, downKeyDown);
		
		// Add to velocity
		this.gameObject.getComponent('RigidBody').velocity
			.add(new THREE.Vector3(-Number(leftKeyDown), 0, 0));
		this.gameObject.getComponent('RigidBody').velocity
			.add(new THREE.Vector3(Number(rightKeyDown), 0, 0));
		this.gameObject.getComponent('RigidBody').velocity
			.add(new THREE.Vector3(0, 0, -Number(upKeyDown)));
		this.gameObject.getComponent('RigidBody').velocity
			.add(new THREE.Vector3(0, 0, Number(downKeyDown)));
	
		// Check if the player is moving
		moving = !this.gameObject.getComponent('RigidBody').velocity.equals(new THREE.Vector3(0, 0, 0));
		
		// Scale Velocity to speed
		this.gameObject.getComponent('RigidBody').velocity.normalize().multiplyScalar(this.speed);

		// Set Rotation according to the player's velocity ONLY IF MOVING
		if(moving)
		{
				this.gameObject.components.creatureAnimation.playWalk();
				this.gameObject.rotation = /*.getComponent('RigidBody').setRotation(*/
					new THREE.Vector3(
						0,
						Math.atan2(-this.gameObject.getComponent('RigidBody').velocity.z, 
								   this.gameObject.getComponent('RigidBody').velocity.x) + Math.PI/2,
						0
					)
				/*)*/;
		}
		else
		{
			if(this.gameObject.components.creatureAnimation.anim == "walk")
			{
				this.gameObject.components.creatureAnimation.stopAnimation();
			}
		}
	/*
		// Animate the component
		if(this.gameObject.getComponent("ANIMATOR_CP"))
		{
			if (!moving && this.gameObject.getComponent("ANIMATOR_CP").nowPlaying == 'walk_cycle') {
				this.gameObject.getComponent("ANIMATOR_CP").stopAnimation();
			} else {
				this.gameObject.getComponent("ANIMATOR_CP").playAnimation('walk_cycle');
			}
		}*/
	};

	/*
		Removes this component
	*/
	this.destroy = function() {
		this.gameObject.removeComponent(this.type);
	};
	
	
	
	this.wouldHitIfMoved

	

};
