// Component for player projectile

/*
	Constructor for the projectile component
	@param {...gameObject} launch - the game object that fired the projectile
	@param {...Vector3} dmg - the damage of the projectile
	@param {...Vector3} vel - the direction of the projectile
*/
var projectile = function(launch, dmg, vel){

	this.type = "projectile";
	this.velocity = vel;
	this.launcher = launch;
	this.damage = dmg;
	this.hitType;
	this.fireType;
	
	

	/*
	Initiates the projectile component to be added to an Object 3D
	*/
	this.init = function(){
		if(this.launcher.getComponent("playerChar"))
		{
			this.hitType = "enemyChar";
			this.fireType = "playerChar";
		}
		else
		{
			this.fireType = "enemyChar";
			this.hitType = "playerChar";
		}
		
		//console.log(this.fireType);
	};

	/*
		Updates the current state of the component
	*/
	this.update = function(gameTime) {
		
		var cm = ENGINE.currentScene.scene.getChildByName("collisionManager");
		
		if(cm)
			cm = cm.getComponent("collisionManager");
		else
			console.log("Could not find Collision Manager");
		
		var isOnScreen = ENGINE.currentScene.mainCamera.getComponent("CAMERTRACKER_CP").IsOnScreen(this.gameObject);
		
		if (!isOnScreen)
			this.destroy();
		
		if(this.gameObject.getComponent("collider") != null)
		{
			if(this.gameObject.getComponent("collider").colliding == true)
			{
				var collider = this.gameObject.getComponent("collider");
				for(var i = 0; i < collider.contacts.length; i++)
				{
					var contact = collider.contacts[i];
					
					//if the collision is not from another player/enemy
					if(!contact.otherCollider.gameObject.getComponent(this.fireType))
					{
						if(contact.otherCollider.gameObject.getComponent(this.hitType))
						{
							var character = contact.otherCollider.gameObject.getComponent(this.hitType);
							//damage colliding character
							character.takeDamage(this.damage);
						}
						
						break;
					}
					
				}
				this.destroy();
			}
			else
			{
				this.gameObject.getComponent("RigidBody").velocity = this.velocity.clone();
				//this.gameObject.position.x += gameTime.deltaTime() * this.velocity.x;
				//this.gameObject.position.z += gameTime.deltaTime() * this.velocity.z;
			}
		}
		
		
	};

	/*
		Removes this component
	*/
	this.destroy = function() {
		//console.log("bullet destroyed");
		
		//Find child light and destroy it
		for(j=0;j<this.gameObject.children.length;j++)
		{
			if(this.gameObject.children[j] instanceof THREE.PointLight)
			{
				console.log("freeing :"+this.gameObject.children[j].name);
				console.log(this.gameObject.children[j]);
				ENGINE.LightPool.FreeLight(this.gameObject.children[j]);
				break;
			}
		}
		
		//Allow launcher to shoot another projectile
		this.launcher.getComponent(this.fireType).shot = false;
		this.gameObject.removeComponent(this.type);
		ENGINE.currentScene.remove(this.gameObject);
	};

	

}
